# @tecnavia/react-native-css

allow to update style of views dinamically by a server side configurations
if you change something, like supported classes update the [wiki](https://wiki.tecnavia.com/doku.php?id=react_native&#react_native_styles)

## Table of Contents    
- [Installation](#installation)   
- [Usage](#usage)   
  - [Provider](#provider)   
  - [Consumer](#consumer)   
  - [Hook](#hook)
  - [Subscription](#subscription)
  - [Direct access](#direct)
- [Methods](#methods)   
  - [Context](#context)   
  - [Others](#others)   
- [Decorators](#decorators)   
- [State](#state)
- [Data](#data)
- [Styles structure](#styles)
- [Throubleshooting](#throubleshooting)

<a name="installation"></a>

## Installation

```sh
npm install git+https://ussvc01.newsmemory.com:3000/tecnavia/react-native-css.git#TAG --save
```

<a name="usage"></a>

## Usage
see example folder to understand all use cases

<a name="provider"></a>

### Provider

put provider on root

```tsx
import Css from "@tecnavia/react-native-css";

<Css.Provider
    value={{
        rnStyles: {},
        activeTheme: undefined,
        isDark: false,
        isPortrait: false,
        isTablet: false
    }}>
    {children}
</Css.Provider>
```

<a name="consumer"></a>

### Consumer
use consumer over each child that you want customize

```tsx
import Css from "@tecnavia/react-native-css";

<Css.Consumer /* @see decorators */>
    {children}
</Css.Consumer>
```

<a name="hook"></a>

### Hook
```tsx
import { useCss } from "@tecnavia/react-native-css";

function Component(){
    const { state, decoratedState } = useCss({ /* @see decorators */ });

    ...
}
```

<a name="subscription"></a>

### Subscription
```tsx
import Css from "@tecnavia/react-native-css";

function Component(){

    useEffect(() => {
        const subscription = Css.subscribe(({ state, decoratedState }) => {
        }, { /* @see decorators */ });
        return () => {
            subscription();
        }
    }, []);

    ...
}
```
<a name="direct"></a>

### Direct access
```tsx
import Css from "@tecnavia/react-native-css";

const { state, decoratedState } = Css.get({ /* @see decorators */ });
...
Css.set(newState);
```

<a name="methods"></a>

## Methods

<a name="context"></a>

### Context
|Name|Description|Return|||
|-|-|-|-|-|
|get|return current Context state|[Data](#data)|[Decorators](#decorators)||
|set|update context||[State](#state)||
|subscribe|callback, called when a context update occurs|function to remove a callback|callback with [Data](#data) params|[Decorators](#decorators)|
|removeAllSubscribers|delete all registered subscribers|||

<a name="others"></a>

### Others
```tsx
import { CssHelper } from "@tecnavia/react-native-css";
```

|Name|Description||
|-|-|-|
|flatten|merge all styles inside array with other styles that comes from classes and decorators||
|getStyle|return array of styles obtained from classes and decorators||
|getStyleObj|call previous getStyle and then flattens to single object||
|getInUseThemeId|activeTheme key, if it is "automatic" the result will be a resolved theme, light or dark||
|getInUseTheme|activeTheme object, if it is "automatic" the result will be a resolved theme, light or dark||
|getActiveThemeId|return current theme id, also "automatic"||
|isActiveThemeDark|return true if current theme is dark||
|mergeRnStyles|merge multiple rn styles structures, first params must be a default structure||
|arrayToObject|merge array of styles into a single object||
|getThemes|return themes object||

<a name="decorators"></a>

## Decorators

|Name|Description|Type|
|-|-|-|
|classes|array of classes to apply|`string[]`|
|toggle|attaches -active and -inactive to each class of above array|`boolean`|
|active|works witch toggle, is used to select the correct class based on activation state|`boolean`|
|cssKey|add -cssKey to each of above classes, also active and inactive|`string`|

<a name="state"></a>

## State
you could set the state by updating Provider value or by static method set;

|Name|Description|Type|
|-|-|-|
|rnStyles|styles object|[Styles structure](#styles)
|activeTheme|active theme otherwise the theme used is the one marked as default |`string`|`undefined`|
|isDark|should be the system dark mode state, if theme is authomatic the library uses this flag to choose correct baseStyles|`boolean`|
|isPortrait|used to select styles based on device orientation|`boolean`|
|isTablet|used to select styles based on device size|`boolean`|
|classes|array of classes to apply|`string[]`|
|toggle|attaches -active and -inactive to each class of above array|`boolean`|
|active|works witch toggle, is used to select the correct class based on activation state|`boolean`|
|cssKey|add -cssKey to each of above classes, also active and inactive|`string`|

<a name="data"></a>

## Data
date returned by Consumer, get, subscription callback and hook
|Name|Description|Type|
|-|-|-|
|state|current context state|[State](#state)
|decoratedState|style obtained by parsing rnStyles from state and a by appling decorators||

<a name="styles"></a>

## Styles structure
the following json describe the root of structure

```json
{
    "base": {
        "styles": {}
    },
    "baseLight": {
        "styles": {}
    },
    "baseDark": {
        "styles": {}
    },
    "themes": {
        "theme1": {},
        //...
        "themeN": {},
        "automathic": {
            "name": "System theme"
        }
    }
}
```

the following json describe styles structure, each class could contains all styles to apply to view in each conditions
remember that you could apply styles attributes supported by nested components, read the lists from:
- [Text](https://reactnative.dev/docs/text-style-props)
- [Image](https://reactnative.dev/docs/image-style-props)
- [View](https://reactnative.dev/docs/view-style-props)
- [Layout](https://reactnative.dev/docs/layout-props)
- [Shadow](https://reactnative.dev/docs/shadow-props)

than could contains some objects with styles applied with specific conditions size, orientation, plaform, os; each of previous conditions could be combined togheter by nesting on inside another multiple times.

* **Platform**
    - phone: applied only on react-native
    - tablet: applied only on react-native
    - mw[0-9]{2,0}: mw means max-width, so the styles are applied to each web devices that has size less than specified number
* **Orientation**
    - portrait
    - landscape   
* **os**
    - ios
    - android
    - windows
* **platform**
    - native: applied if we are working with react-native
    - web: applied if we are working with react or react-native-web

```json
{
    "class1": {
        //...style...
        "portrait": {
            //...style...,
            "android": {
                //...style...
                //etc...
            }
            //etc..
        },
        "android": {
            //...style...
            //etc..
        }
    },
    //...
    "classN": {

    }
}

the following json instead describe theme structure
```json
{
    "name": "Theme name",
    "default": 0,
    "dark": 0,
    "automatic": 0,
    "styles": {}
}
```

|Name|Default|Required|Description|
|-|:-:|:-:|-|
|name||true|description name|
|default|0|false|only one theme in the list must be set like default theme, if multiple theme are defaults the library uses the first in the list|
|dark|0|false|if 1 the theme will be merged with baseDark otherwise with baseLight|
|automatic|0|false|only two theme should be authomatic, one light and one dark, because are applied based on current system mode|
|styles||false|if empty the styles are applied just from based styles|

<a name="throubleshooting"></a>

## Throubleshooting

- Don't use **Css.Consumer** component inside a touchable i.e. **TouchableOpacity** etc... because otherwise the element is not clickable, this is caused by a **0,0** size assigned to touchable element. To solve the issue you could use the **hook** or simply the exposed method **getStyleObj**
