import { StyleSheet } from "react-native";
import { isAndroid, isIOS, isWeb, isWindows } from "./platform";
export let CssManager;
(function (_CssManager) {
  /**
   * remove all keys mw from object
   * mw keys are related to screen width
   * @param {*} css
   * @returns
   */
  function maxWidthCss() {
    let css = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    const mw = {};
    const rest = {};
    Object.keys(css).forEach(key => {
      if (key.match(/mw[0-9]{2,}/)) {
        mw[key] = css[key];
      } else {
        rest[key] = css[key];
      }
    });
    return {
      mw,
      rest
    };
  }

  /**
   * return a style with all mw rules applied based on device width
   * mw means max-width so return all styles with mw greather than device width
   * @param {*} mw
   * @returns
   */
  function traverseAndFlattenMaxWidthCss(state, mw) {
    const wSize = state.window;
    let mwStyle = {};
    if (wSize && wSize.width) {
      const constraints = [];
      const mwKeys = Object.keys(mw);
      for (let i = 0; i < mwKeys.length; i++) {
        const key = mwKeys[i];
        const constraint = parseInt(key.replace("mw", ""), 10);
        if (wSize.width < constraint) {
          constraints.push(`mw${constraint}`);
        }
      }
      constraints.forEach(constraint => {
        const style = traverseAndFlattenCss(state, mw[constraint]);
        mwStyle = {
          ...mwStyle,
          ...style
        };
      });
    }
    return mwStyle;
  }

  /**
   * this method put all styles customized for a device (tablet|phone, ios|android, landscape|portrait) in a flat object
   * @param {*} css
   */
  function traverseAndFlattenCss(state, css) {
    if (css !== undefined) {
      const {
        tablet,
        phone,
        ios,
        android,
        native,
        web,
        windows,
        landscape,
        portrait,
        ...cssRest
      } = css;
      const {
        mw,
        rest
      } = maxWidthCss(cssRest);
      let deviceStyle = {};
      if (state.isTablet) {
        deviceStyle = traverseAndFlattenCss(state, tablet);
      } else {
        deviceStyle = traverseAndFlattenCss(state, phone);
      }
      let maxWidthStyle = {};
      //?is usefull also inside mobile apps?
      if (isWeb()) {
        maxWidthStyle = traverseAndFlattenMaxWidthCss(state, mw);
      }
      let nativeStyle = {};
      if (!isWeb()) {
        nativeStyle = traverseAndFlattenCss(state, native);
      }
      let osStyle = {};
      if (isIOS()) {
        osStyle = traverseAndFlattenCss(state, ios);
      } else if (isAndroid()) {
        osStyle = traverseAndFlattenCss(state, android);
      } else if (isWeb()) {
        osStyle = traverseAndFlattenCss(state, web);
      } else if (isWindows()) {
        osStyle = traverseAndFlattenCss(state, windows);
      }
      let orientationStyle = {};
      if (state.isPortrait) {
        orientationStyle = traverseAndFlattenCss(state, portrait);
      } else {
        orientationStyle = traverseAndFlattenCss(state, landscape);
      }
      return {
        ...rest,
        ...deviceStyle,
        ...maxWidthStyle,
        ...nativeStyle,
        ...osStyle,
        ...orientationStyle
      };
    }
    return {};
  }
  function stylesArrayToObject(styles) {
    let ignoreKeys = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    const style = {};
    styles.forEach(item => {
      Object.keys(item).forEach(key => {
        if (!ignoreKeys.includes(key)) {
          style[key] = item[key];
        }
      });
    });
    return style;
  }

  /**
   * @see IRnStyles
   * @param {*} name
   * @param {*} theme
   */
  function getStyleByCssName(state, name) {
    const {
      rnStyles
    } = state;
    const styles = [];
    //add default app styles
    if (rnStyles && name) {
      //we have a default style with all base styles
      //also user could select a specific style with some customization applied over default
      let userStyles;
      const {
        themes,
        base,
        baseLight,
        baseDark
      } = rnStyles;
      const themeInUse = getInUseThemeId(state);
      if (typeof themes === "object" && themeInUse !== undefined) {
        userStyles = themes[themeInUse];
      }

      //than add style from base theme
      if (base && base.styles) {
        styles.push(traverseAndFlattenCss(state, base.styles[name]));
      }

      //than add style from base appearance theme
      if (userStyles && userStyles.dark === 1) {
        if (baseDark && baseDark.styles) {
          styles.push(traverseAndFlattenCss(state, baseDark.styles[name]));
        }
      } else {
        if (baseLight && baseLight.styles) {
          styles.push(traverseAndFlattenCss(state, baseLight.styles[name]));
        }
      }
      //add user theme if the user select something inside option page, otherwise add default selected theme
      if (userStyles && userStyles.styles) {
        styles.push(traverseAndFlattenCss(state, userStyles.styles[name]));
      }
    }
    return stylesArrayToObject(styles);
  }
  function applyCustomClasses(props) {
    const {
      classes,
      toggle,
      active,
      cssKey
    } = props;
    const newClasses = [];
    if (classes && classes.length) {
      newClasses.push(...classes);
      //add automatically active and inactive classes after all classes available
      if (toggle) {
        for (let i = 0; i < classes.length; i++) {
          if (active) {
            newClasses.push(`${classes[i]}-active`);
          } else {
            newClasses.push(`${classes[i]}-inactive`);
          }
        }
      }
      //add automatically key custom classes after all classes available
      if (cssKey) {
        const {
          length
        } = newClasses;
        for (let i = 0; i < length; i++) {
          newClasses.push(`${newClasses[i]}-${cssKey}`);
        }
      }
    }
    return newClasses;
  }
  function getStylesByCssClasses(state, classes) {
    let attributes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    const newClasses = applyCustomClasses({
      classes,
      ...attributes
    });
    if (Array.isArray(newClasses)) {
      return newClasses.map(css => {
        return getStyleByCssName(state, css);
      });
    }
    return [];
  }
  function flatten() {
    let state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    let styles = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    let classes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
    let attributes = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
    return StyleSheet.flatten([...styles, getStyle(state, classes, attributes)]);
  }
  _CssManager.flatten = flatten;
  function arrayToObject(styles, ignoreKeys) {
    return stylesArrayToObject(styles, ignoreKeys);
  }
  _CssManager.arrayToObject = arrayToObject;
  function getStyle() {
    let state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    let classes = arguments.length > 1 ? arguments[1] : undefined;
    let attributes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    const {
      ignoreKeys
    } = attributes;
    const styles = getStylesByCssClasses(state, classes, attributes);
    return stylesArrayToObject(styles, ignoreKeys);
  }
  _CssManager.getStyle = getStyle;
  function getStyleObj() {
    let context = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    let classes = arguments.length > 1 ? arguments[1] : undefined;
    let attributes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    const {
      ignoreKeys
    } = attributes;
    const styles = getStylesByCssClasses(context, classes, attributes);
    return stylesArrayToObject(styles, ignoreKeys);
  }
  _CssManager.getStyleObj = getStyleObj;
  function getThemes() {
    var _context$rnStyles;
    let context = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return context !== null && context !== void 0 && (_context$rnStyles = context.rnStyles) !== null && _context$rnStyles !== void 0 && _context$rnStyles.themes ? Object.keys(context.rnStyles.themes).map(key => ({
      ...context.rnStyles.themes[key],
      key
    })) : [];
  }
  _CssManager.getThemes = getThemes;
  function isActiveThemeDark() {
    var _getInUseTheme;
    let context = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return ((_getInUseTheme = getInUseTheme(context)) === null || _getInUseTheme === void 0 ? void 0 : _getInUseTheme.dark) === 1;
  }
  _CssManager.isActiveThemeDark = isActiveThemeDark;
  function getInUseTheme() {
    var _context$rnStyles2;
    let context = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    const id = getInUseThemeId(context);
    return (_context$rnStyles2 = context.rnStyles) !== null && _context$rnStyles2 !== void 0 && _context$rnStyles2.themes && id ? context.rnStyles.themes[id] : undefined;
  }
  _CssManager.getInUseTheme = getInUseTheme;
  function getInUseThemeId() {
    let {
      rnStyles = {},
      isDark,
      activeTheme
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    const {
      themes
    } = rnStyles;
    if (typeof themes === "object") {
      //avoid server side issue, someone could add a key like a string that is not an object
      const validThemesKeys = Object.keys(themes).filter(key => typeof themes[key] === "object");
      if (validThemesKeys.length > 0) {
        //determine default theme
        //not override if user theme is set before default
        if (!activeTheme) {
          activeTheme = validThemesKeys.find(key => {
            var _themes$key;
            return ((_themes$key = themes[key]) === null || _themes$key === void 0 ? void 0 : _themes$key.default) === 1;
          });
        }
        if (activeTheme === "automatic") {
          activeTheme = validThemesKeys.filter(key => {
            var _themes$key2;
            return key !== "automatic" && ((_themes$key2 = themes[key]) === null || _themes$key2 === void 0 ? void 0 : _themes$key2.automatic) === 1;
          }).find(key => {
            var _themes$key3;
            const isThemeDark = ((_themes$key3 = themes[key]) === null || _themes$key3 === void 0 ? void 0 : _themes$key3.dark) === 1;
            //if theme is dark and system is dark or theme is light and system is light
            return isThemeDark === isDark;
          });
        }
        if (!activeTheme) {
          //no theme set, pick first valid
          activeTheme = validThemesKeys[0];
        }
      }
    }
    return activeTheme;
  }
  _CssManager.getInUseThemeId = getInUseThemeId;
  function getActiveThemeId() {
    let {
      rnStyles = {},
      activeTheme
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    if (activeTheme) {
      return activeTheme;
    }
    const {
      themes
    } = rnStyles;
    if (typeof themes === "object") {
      return Object.keys(themes).find(key => {
        var _themes$key4;
        return typeof themes[key] === "object" && ((_themes$key4 = themes[key]) === null || _themes$key4 === void 0 ? void 0 : _themes$key4.default) === 1;
      });
    }
    return undefined;
  }
  _CssManager.getActiveThemeId = getActiveThemeId;
})(CssManager || (CssManager = {}));
//# sourceMappingURL=css-manager.js.map